@@ -1,6 +1,6 @@
 from contextlib import contextmanager
 from ..base import multipath
-from ..errors import StorageModelFindError, MultipathDaemonTimeoutError
+from ..errors import StorageModelFindError, MultipathDaemonTimeoutError, DeviceDisappeared
 from infi.pyutils.lazy import cached_method
 from .block import LinuxBlockDeviceMixin
 import itertools
@@ -15,11 +15,18 @@ class LinuxNativeMultipathBlockDevice(LinuxBlockDeviceMixin, multipath.Multipath
         self.sysfs_device = sysfs_device
         self.multipath_object = multipath_object
 
+    @contextmanager
     def asi_context(self):
-        for path in self.get_paths():
-            if path.get_state() == "up":
-                return path.asi_context()
-        raise StorageModelFindError("cannot find an active path to open SCSI generic device") # pylint: disable=W0710
+        import os
+        from infi.asi.unix import OSFile
+        from infi.asi.linux import LinuxIoctlCommandExecuter
+
+        handle = OSFile(os.open(self.get_block_access_path(), os.O_RDWR))
+        executer = LinuxIoctlCommandExecuter(handle)
+        try:
+            yield executer
+        finally:
+            handle.close()
 
     def _is_there_atleast_one_path_up(self):
         return bool(filter(lambda path: path.get_state() == "up", self.get_paths()))
@@ -61,9 +68,9 @@ class LinuxPath(multipath.Path):
         import os
         from infi.asi.unix import OSFile
         from infi.asi import create_platform_command_executer
-
+        from .scsi import SG_TIMEOUT_IN_MS
         handle = OSFile(os.open(os.path.join("/dev", self.sysfs_device.get_scsi_generic_device_name()), os.O_RDWR))
-        executer = create_platform_command_executer(handle)
+        executer = create_platform_command_executer(handle, timeout=SG_TIMEOUT_IN_MS)
         try:
             yield executer
         finally:
@@ -107,7 +114,10 @@ class LinuxNativeMultipathModel(multipath.NativeMultipathModel):
             logger.info("MultipathD is not running")
             return []
 
-        devices = self._get_list_of_active_devices(client)
+        try:
+            devices = self._get_list_of_active_devices(client)
+        except IOError:
+            raise DeviceDisappeared()
         result = []
         logger.debug("Got {} devices from multipath client".format(len(devices)))
         for mpath_device in devices:
